
-- Creates our view and hooks the required events for the addon to load properly.
function amtr.init()

	bombTracker.init();
	bombTracker.setTarget("target");
	
	powerTalentTracker.init();
	
	amtr.updateModel(); -- createModel would be better?

	amtr.createView();
	amtr.layoutView(bombTracker.spellName, powerTalentTracker.spellName);


	-- move these to a WoWProxy or WowEventProxy and keep any game interaction there
	amtr.frame:SetScript("OnEvent", amtr.OnEvent);

	amtr.frame:RegisterEvent("ACTIVE_TALENT_GROUP_CHANGED");
	amtr.frame:RegisterEvent("LEARNED_SPELL_IN_TAB");
	amtr.frame:RegisterEvent("PLAYER_ALIVE");
	amtr.frame:RegisterEvent("VARIABLES_LOADED");

	-- http://wowprogramming.com/utils/artbrowser/Interface/ICONS
	-- Interface\ICONS\spell_arcane_arcane01    .png
	--name, rank, icon, cost, isFunnel, powerType, castTime, minRange, maxRange = GetSpellInfo(SPELL_FROST_BOMB_ID);
	--print("Arcane Charge " .. name .. " icon " .. icon);
end

-- Updates our view every frame.
function amtr.onUpdate(elapsed)
	amtr.updateModel();
	amtr.updateView();
end


-- Enables and displays the addon on startup and spec changes.
function amtr.enable()

	amtr.frame:Show();
	amtr.frame:SetScript("OnUpdate", amtr.onUpdate);
	--amtr.frame:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED");
	amtr.frame:RegisterEvent("PLAYER_TARGET_CHANGED");
	amtr.frame:RegisterEvent("MODIFIER_STATE_CHANGED");
	amtr.frame:RegisterEvent("PLAYER_REGEN_ENABLED");
	amtr.frame:RegisterEvent("PLAYER_REGEN_DISABLED");
	amtr.frame:RegisterEvent("PLAYER_TALENT_UPDATE");

	bombTracker.detectBombType();
	powerTalentTracker.detectTalent();

	-- It's not clear in the view that layoutView gets calledn onEnable
	amtr.layoutView(bombTracker.spellName, powerTalentTracker.spellName);

	local inCombat = UnitAffectingCombat("player");
	if (inCombat) then
		amtr.onCombatStarted();
	else
		amtr.onCombatEnded();
	end

end

function amtr.disable()
	amtr.frame:Hide();
	amtr.frame:SetScript("OnUpdate", nil);
	--amtr.frame:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED");
	amtr.frame:UnregisterEvent("PLAYER_TARGET_CHANGED");
	amtr.frame:UnregisterEvent("MODIFIER_STATE_CHANGED");
	amtr.frame:UnregisterEvent("PLAYER_REGEN_ENABLED");
	amtr.frame:UnregisterEvent("PLAYER_REGEN_DISABLED");
	amtr.frame:UnregisterEvent("PLAYER_TALENT_UPDATE");

end


function amtr.isArcaneSpec()
	local currentSpec = GetSpecialization();
	if (currentSpec == nil) then	-- no specs
		return false;
	end
	
	local id = GetSpecializationInfo(currentSpec);
	return (id == SPEC_ARCANE);
end

-- Update the frame width and saves the fontSize to config.
--[[
function amtr.setFontSize( fontSize )
	amtrConfig.fontSize = fontSize;
	amtr.layoutView();
end
--]]
function amtr.OnEvent(frame, event, ...)
	if (event == "COMBAT_LOG_EVENT_UNFILTERED") then
	
	elseif (event == "PLAYER_TARGET_CHANGED") then
	
	elseif (event == "PLAYER_ALIVE" or event == "ACTIVE_TALENT_GROUP_CHANGED" or event == "LEARNED_SPELL_IN_TAB" or event == "PLAYER_TALENT_UPDATE") then

		if (amtr.isArcaneSpec()) then
			amtr.enable();
		else
			amtr.disable();
		end


	elseif (event == "VARIABLES_LOADED") then

		if (amtr.isArcaneSpec()) then
			amtr.enable();
		else
			amtr.disable();
		end
	elseif (event == "PLAYER_REGEN_ENABLED") then
		amtr.onCombatEnded();
	elseif (event == "PLAYER_REGEN_DISABLED") then
		amtr.onCombatStarted();
		--[[
	elseif (event == "MODIFIER_STATE_CHANGED" and amtr.frame:IsShown()) then
		local key, state = ...
		if (key == "LCTRL" or key == "RCTRL") then
			if (state == 1) then
				amtr.frame:EnableMouse(true);
			else
				amtr.frame:EnableMouse(false);
				amtr.frame:StopMovingOrSizing();
			end
		end
	--]]
	end

end

function amtr.onCombatStarted()

	if ( amtr.isArcaneSpec() ) then
		amtr.frame:Show();
	else
		amtr.frame:Hide();
	end
end

function amtr.onCombatEnded()
	if (amtrConfig.hideOutOfCombat == true) then
		if ( amtr.isArcaneSpec() ) then
			amtr.frame:Hide();
		end
	else
		amtr.frame:Show();
	end
end


function amtr.getBombDebuffInfo( unit )

	buffName, buffRank, buffTexture, buffApplications, school, duration, timeLeft, unitCaster, buffId, isStealable, buffId2 = UnitDebuff(unit, spellInfoCached.livingBomb, nil, "PLAYER");
	if (buffName ~= nil) then
		return duration, (timeLeft - GetTime()), SPELL_LIVING_BOMB_ID;
	end

	buffName, buffRank, buffTexture, buffApplications, school, duration, timeLeft, unitCaster, buffId, isStealable, buffId2 = UnitDebuff(unit, spellInfoCached.netherTempest, nil, "PLAYER");
	if (buffName ~= nil) then
		return duration, (timeLeft - GetTime()), SPELL_NETHER_TEMPEST_ID;
	end

	buffName, buffRank, buffTexture, buffApplications, school, duration, timeLeft, unitCaster, buffId, isStealable, buffId2 = UnitDebuff(unit, spellInfoCached.frostBomb, nil, "PLAYER");
	if (buffName ~= nil) then
		return duration, (timeLeft - GetTime()), SPELL_FROST_BOMB_ID;
	end

	return nil, nil;
end

function formatNumberComma(amount)
  local formatted = amount
  while true do  
    formatted, k = string.gsub(formatted, "^(-?%d+)(%d%d%d)", '%1,%2')
    if (k==0) then
      break
    end
  end
  return formatted
end

function booleanToString(value)
	if (value) then
		return "true";
	else
		return "false";
	end
end


SLASH_amtr1, SLASH_amtr2 = "/amtr", "/arcane";
SlashCmdList.amtr = function(arg)

	local tokens = {};
	for token in string.gmatch(arg, "[^%s]+") do
		table.insert(tokens, token)
	end
	local command = tokens[1];

	if (command == "debug") then
		amtrConfig.debug = (amtrConfig.debug == false);
		print(PRINT_COLOUR .. "amtr.debug = |r" .. booleanToString(amtrConfig.debug));

	elseif (command == "combat") then
		amtrConfig.hideOutOfCombat = (amtrConfig.hideOutOfCombat == false);
		print(PRINT_COLOUR .. "amtr.hideOutOfCombat = |r" .. booleanToString(amtrConfig.hideOutOfCombat));

		local isInCombat = UnitAffectingCombat("player");
		if (isInCombat == nil) then
			amtr.onCombatEnded();
		else
			amtr.onCombatStarted();
		end

	elseif (command == "reset") then
		amtr.frame:SetWidth(DEFAULT_WIDTH);
		amtr.frame:SetHeight(DEFAULT_HEIGHT);
		amtr.layoutView(bombTracker.spellName);

	elseif (command == "lock") then
		amtrConfig.lockWindow = (amtrConfig.lockWindow == false);
		print(PRINT_COLOUR .. "amtr.lockWindow = |r" .. booleanToString(amtrConfig.lockWindow));

	elseif (tonumber(arg)) then
		amtrConfig.minMana = tonumber(arg);
		print(PRINT_COLOUR .. "amtr.minMana = |r" .. tonumber(amtrConfig.minMana));

	else	-- Print help
		--print(PRINT_COLOUR .. "/amtr ignite = " .. booleanToString(amtrConfig.displayIgnite) .. "|r - Display Ignite amount instead of predicted Combustion amount.");
		print(PRINT_COLOUR .. "/amtr <minimum mana> = " .. amtrConfig.minMana .. "|r - Set the Mana Bar's lower limit using either a percentage or a fixed amount.");
		--print(PRINT_COLOUR .. "/amtr reset|r - Reset the font size.");
		print(PRINT_COLOUR .. "/amtr combat = " .. booleanToString(amtrConfig.hideOutOfCombat) .. "|r - Only shows Arcanometer when in combat.");
		print(PRINT_COLOUR .. "/amtr reset|r - Reset size and position of Arcanometer.");
		print(PRINT_COLOUR .. "/amtr lock = " .. booleanToString(amtrConfig.lockWindow) .. "|r - Toggle the window size and position lock.");
		--print(PRINT_COLOUR .. "/amtr debug = " .. booleanToString(amtrConfig.debug) .. "|r - Print Combustion ticks in the chat panel.");
	end
end

amtr:init()
