--[[
	Utility class to retrieve info about any Mage bombs on an enemy target.
--]]

-- Global reference
bombTracker = {};
bombTracker.spellName = nil;-- Set to currently known bomb spell name by detectBombType.
bombTracker.target = nil;		-- Track bombs on this unit.

SPELL_LIVING_BOMB_ID = 82885;
SPELL_LIVING_BOMB_TICK_ID = 44457
SPELL_NETHER_TEMPEST_ID = 114954;
SPELL_NETHER_TEMPEST_TICK_ID = 114923;
SPELL_FROST_BOMB_ID = 64627;
SPELL_FROST_BOMB_TICK_ID = 112948;
-- Localized names.
local spellNames = {
	livingBomb = GetSpellInfo(SPELL_LIVING_BOMB_ID),
	netherTempest = GetSpellInfo(SPELL_NETHER_TEMPEST_ID),
	frostBomb = GetSpellInfo(SPELL_FROST_BOMB_ID)
}

function bombTracker.init()
	bombTracker.detectBombType();
end

-- Detect which bomb the player currently has.
-- Needs to be called if a player's talents change.
function bombTracker.detectBombType()

	if (bombTracker.hasSpell(SPELL_LIVING_BOMB_ID)) then
		bombTracker.spellName = spellNames.livingBomb;--SPELL_LIVING_BOMB_ID;
	elseif (bombTracker.hasSpell(SPELL_NETHER_TEMPEST_ID)) then
		bombTracker.spellName = spellNames.netherTempest;
	elseif (bombTracker.hasSpell(SPELL_FROST_BOMB_ID)) then
		bombTracker.spellName = spellNames.frostBomb;
	else
		bombTracker.spellName = nil;
		--print("No Mage bomb talented!");
	end

end

function bombTracker.hasSpell(spellId)
	local name = GetSpellInfo(spellId);
	local link = GetSpellLink(name);
	if (link ~= nil) then
		return true;
	else
		return false;
	end
end

-- Most likely always set to "target". But could be a guid too.
function bombTracker.setTarget( target )
	bombTracker.target = target;
end

-- Returns the duration, time left, spell id and cooldown left for frost bomb.
-- if a bomb exists on the set target.
-- returns nil if no bomb was found.
function bombTracker.getInfo()

	if (bombTracker.spellName == nil) then	-- No target selected or no bomb talented.
		return nil, nil, nil, nil;
	end

	duration, timeLeft = bombTracker.getDebuffTimeLeft(bombTracker.target, bombTracker.spellName);
	local cdTimeLeft = 0;
	if (bombTracker.spellName == spellNames.frostBomb) then
		local cdStart, cdDuration = GetSpellCooldown(SPELL_FROST_BOMB_ID);
		if (cdStart > 0) then
			timeLeft = duration - (GetTime() - cdStart);
		end
	end

	return duration, timeLeft, bombTracker.spellName, cdTimeLeft;

end

-- UnitDebuff does not work unless we use the name. Otherwise we have to loop through all spells.
function bombTracker.getDebuffTimeLeft(target, spellName)
	buffName, _, _, _, _, duration, timeLeft = UnitDebuff(target, spellName, nil, "PLAYER");
	if (buffName ~= nil) then
		return duration, timeLeft - GetTime();
	else
		return nil, nil
	end
end